/***********************************************************************
 *              Berliner Uhr
 *            ================
 *            Uwe Berger; 2009
 * 
 * 
 * Version mit (nativ) X.
 * 
 * ---------
 * Have fun!
 * 
 **********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <X11/keysym.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define WINDOW_WIDTH	170
#define WINDOW_HEIGHT	WINDOW_WIDTH
#define BORDER			3
#define	DY				WINDOW_HEIGHT/5
#define DX				WINDOW_WIDTH

/* Funktionsprototypen */
static int create_window (void);
static void process_event (XEvent report);
static void eventloop (void);
static void close_window (void);
static void draw_graphics (void);
static void setcolourvianame (GC, char *);

/* Globale Variablen */
static unsigned int width, height;
static Display *display;
static int screen;
static int depth;
static Window win;

int now_s, now_m, now_h;
int old_m=60;
int refresh=1;
time_t now;
struct tm *now_tm;


//**********************************************************************
static int create_window (void) {
  char *window_name = "Berliner Uhr";
  char *icon_name   = "window";
  static XSizeHints size_hints;
  Window rootwin;

  width = WINDOW_WIDTH;
  height = WINDOW_HEIGHT;

  /* X-Sitzung öffnen */
  display = XOpenDisplay (NULL);
  /* Fehlerüberprüfung */
  if (display == NULL) {
    printf ("Verbindung zum X-Server fehlgeschlagen?!?\n");
    exit(EXIT_FAILURE);
  }
  /* Standardbildschirm eines Displays */
  screen = XDefaultScreen (display);
  /* Standardtiefe des screen - Anzahl der Ebenen */
  depth = XDefaultDepth (display, screen);
  rootwin = RootWindow (display, screen);
  win = XCreateSimpleWindow ( display, rootwin, 10, 10,
                              width, height, 5,
                              BlackPixel (display, screen),
                              WhitePixel (display, screen)  );
  size_hints.flags = PSize | PMinSize | PMaxSize;
  size_hints.min_width  = width;
  size_hints.max_width  = width;
  size_hints.min_height = height;
  size_hints.max_height = height;
  XSetStandardProperties ( display, win, window_name, icon_name,
                           None, 0, 0, &size_hints);
  XSelectInput (display, win,
                ButtonPressMask | KeyPressMask | ExposureMask);
  XMapWindow (display, win);
  return 1;
}

//**********************************************************************
static void close_window (void) {
  XCloseDisplay (display);
}

//**********************************************************************
static void eventloop (void) {
  XEvent xev;
  int num_events;

  XFlush (display);
  num_events = XPending (display);
  while ((num_events != 0)) {
    num_events--;
    XNextEvent (display, &xev);
    process_event (xev);
  }
}

//**********************************************************************
static void process_event (XEvent report) {

	if (report.xexpose.count == 0) {
		refresh=1;
		draw_graphics ();
	}
}

//**********************************************************************
static void draw_graphics (void) {
	GC mygc;                      /* neu: Grafikkontext */
	unsigned long black, white;   /* Nummern der Farben */
	int i;

	// aktuelle Uhrzeit
	now = time (NULL);
	now_tm	= localtime (&now);
    now_s	= now_tm->tm_sec;
	now_m	= now_tm->tm_min;
	now_h	= now_tm->tm_hour;

	XMapRaised (display, win);
	/* Farbnummern besorgen */
	black = BlackPixel (display, screen);
	white = WhitePixel (display, screen);

	/* Graphical Context erzeugen, Farben eintragen */
	mygc = XCreateGC (display, win, 0, 0);
	XSetForeground (display, mygc, black);
	XSetBackground (display, mygc, white);

	// Fensterhintergrund schwarz
	//setcolourvianame (mygc,  "black");
	//XFillRectangle (display, win, mygc, 0, 0, width, height);
	// Sekundenanzeige
	if(now_s % 2) setcolourvianame (mygc, "gray8"); else setcolourvianame (mygc, "yellow");
	XFillArc (display, win, mygc, \
	          width/2-(DY-2*BORDER)/2, 0+BORDER, 
	          DY-2*BORDER, \
	          DY-2*BORDER, \
	          0, 360*64);
	
	// hat sich die Minute geaendert?
	if ((old_m==now_m)&&!refresh) return;
	old_m = now_m; 
	refresh=0;

	// Fensterhintergrund schwarz
	setcolourvianame (mygc,  "black");
	XFillRectangle (display, win, mygc, 0, 0, width, height);
	// Sekundenanzeige
	if(now_s % 2) setcolourvianame (mygc, "gray8"); else setcolourvianame (mygc, "yellow");
	XFillArc (display, win, mygc, \
	          width/2-(DY-2*BORDER)/2, 0+BORDER, 
	          DY-2*BORDER, \
	          DY-2*BORDER, \
	          0, 360*64);
	// Sonderlocke 00:00 Uhr...
	if ((now_h==0)&&(now_m==0)) now_h = 24;
	// 5er Stunden
	for (i=0; i<4; i++) {
		if (now_h/((i+1)*5)) setcolourvianame (mygc, "red"); else setcolourvianame (mygc, "gray8");
		XFillRectangle (display, win, mygc, \
		                DX/4*i+BORDER, \
		                1*DY+BORDER, \
		                DX/4-BORDER, \
		                DY-BORDER);
	}	
	// 1er Stunden
	for (i=0; i<4; i++) {
		if ((now_h%5)>i) setcolourvianame (mygc, "red"); else setcolourvianame (mygc, "gray8");
		XFillRectangle (display, win, mygc, \
		                DX/4*i+BORDER, \
		                2*DY+BORDER, \
		                DX/4-BORDER, \
		                DY-BORDER);
	}	
	// 5er Minuten
	for (i=0; i<11; i++) {
		if (now_m/((i+1)*5)) {
			// jedes dritte Zeichen in Rot...
			if ((i+1)%3) setcolourvianame (mygc, "yellow"); else setcolourvianame (mygc, "red");
			 
		} else setcolourvianame (mygc, "gray8");
		XFillRectangle (display, win, mygc, \
		                DX/11*i+BORDER, \
		                3*DY+BORDER, \
		                DX/11-BORDER, \
		                DY-BORDER);
	}
	// 1er Minuten
	for (i=0; i<4; i++) {
		if ((now_m%5)>i) setcolourvianame (mygc, "yellow"); else setcolourvianame (mygc, "gray8");
		XFillRectangle (display, win, mygc, \
		                DX/4*i+BORDER, \
		                4*DY+BORDER, \
		                DX/4-BORDER, \
		                DY-BORDER);
	}	
	XFlush (display);
	XFreeGC (display, mygc);
}

//**********************************************************************
static void setcolourvianame (GC myGC, char *colourname) {
  /* genaue RGB  Komponenten der Farbe */
  XColor rgbcolour;
  /* am naheste liegende RGB-Farben der Hardware */
  XColor hwarecolour;
  int status;
  /* Colormap */
  Colormap  myColourmap;
  int myDepth;

  myDepth = DefaultDepth (display, screen);
  myColourmap = DefaultColormap (display, screen);
  screen = DefaultScreen (display);
  if (myDepth > 1) {
    status = XLookupColor (
       display, myColourmap,colourname,&rgbcolour,&hwarecolour );
    if (status != 0) {
      /* Farben für die Color map reservieren */
      status = XAllocColor ( display, myColourmap, &hwarecolour);
      if (status != 0) {
        /* Zeichenfarbe setzen */
        XSetForeground (display, myGC, hwarecolour.pixel );
      }
      else {
        printf ("Kann Farbzelle für %s nicht ermitteln\n",
           colourname);
        printf ("Verwende stattdessen Schwarz\n");
        XSetForeground(display, myGC,BlackPixel(display,screen));
      }
    }
    else {
      printf ("Kann Farbe %s nicht finden\n", colourname);
      printf ("Verwende stattdessen Schwarz\n");
      XSetForeground(display, myGC, BlackPixel(display, screen));
    }
  }
  else {
    printf("Keine Farbe in in dieser Tiefe:%d ?!\n", myDepth);
    printf ("Verwende stattdessen Schwarz \n");
    XSetForeground (display, myGC, BlackPixel(display, screen));
  }
}

//**********************************************************************
//**********************************************************************
//**********************************************************************
int main (int argc, char **argv) {

  create_window ();
  while (1) {
    eventloop ();
    draw_graphics();
    sleep(1);
  }
  close_window ();
  return EXIT_SUCCESS;
}
